@@ -1,6 +1,7 @@
 import warnings
 
 import django
+from collections import Counter
 from django.contrib.admin.utils import NestedObjects
 from django.db import models, router
 from django.db.models import UniqueConstraint
@@ -10,6 +11,7 @@ from .config import (
     FIELD_NAME,
     HARD_DELETE,
     HARD_DELETE_NOCASCADE,
+    DELETED_BY_CASCADE_FIELD_NAME,
     NO_DELETE,
     SOFT_DELETE,
     SOFT_DELETE_CASCADE,
@@ -52,6 +54,21 @@ class SafeDeleteModel(models.Model):
         DateTimeField set to the moment the object was deleted. Is set to
         ``None`` if the object has not been deleted.
 
+    :attribute deleted_by_cascade:
+        BooleanField set True whenever the object is deleted due cascade operation called by delete
+        method of any parent Model. Default value is False. Later if its parent model calls for
+        cascading undelete, it will restore only child classes that were also deleted by a cascading
+        operation (deleted_by_cascade equals to True), i.e. all objects that were deleted before their
+        parent deletion, should keep deleted if the same parent object is restored by undelete method.
+
+        If this behavior isn't desired, class that inherits from SafeDeleteModel can override this
+        attribute by setting it as None: overriding model class won't have its ``deleted_by_cascade``
+        field and won't be restored by cascading undelete even if it was deleted by a cascade operation.
+
+        >>> class MyModel(SafeDeleteModel):
+        ...     deleted_by_cascade = None
+        ...     my_field = models.TextField()
+
     :attribute _safedelete_policy: define what happens when you delete an object.
         It can be one of ``HARD_DELETE``, ``SOFT_DELETE``, ``SOFT_DELETE_CASCADE``, ``NO_DELETE`` and ``HARD_DELETE_NOCASCADE``.
         Defaults to ``SOFT_DELETE``.
@@ -103,6 +120,7 @@ class SafeDeleteModel(models.Model):
             if getattr(self, FIELD_NAME) and self.pk:
                 was_undeleted = True
             setattr(self, FIELD_NAME, None)
+            setattr(self, DELETED_BY_CASCADE_FIELD_NAME, False)
 
         super(SafeDeleteModel, self).save(**kwargs)
 
@@ -125,15 +143,19 @@ class SafeDeleteModel(models.Model):
 
         assert getattr(self, FIELD_NAME)
         self.save(keep_deleted=False, **kwargs)
+        undeleted_counter = Counter({self._meta.label: 1})
 
         if current_policy == SOFT_DELETE_CASCADE:
-            for related in related_objects(self):
+            for related in related_objects(self, only_deleted_by_cascade=True):
                 if is_safedelete_cls(related.__class__) and getattr(related, FIELD_NAME):
-                    related.undelete()
+                    _, undelete_response = related.undelete(**kwargs)
+                    undeleted_counter.update(undelete_response)
+
+        return sum(undeleted_counter.values()), dict(undeleted_counter)
 
     def delete(self, force_policy=None, **kwargs):
         # To know why we need to do that, see https://github.com/makinacorpus/django-safedelete/issues/117
-        self._delete(force_policy, **kwargs)
+        return self._delete(force_policy, **kwargs)
 
     def _delete(self, force_policy=None, **kwargs):
         """Overrides Django's delete behaviour based on the model's delete policy.
@@ -145,19 +167,24 @@ class SafeDeleteModel(models.Model):
         current_policy = self._safedelete_policy if (force_policy is None) else force_policy
 
         if current_policy == NO_DELETE:
-            pass
+            return (0, {})
         elif current_policy == SOFT_DELETE:
-            self.soft_delete_policy_action(**kwargs)
+            return self.soft_delete_policy_action(**kwargs)
         elif current_policy == HARD_DELETE:
-            self.hard_delete_policy_action(**kwargs)
+            return self.hard_delete_policy_action(**kwargs)
         elif current_policy == HARD_DELETE_NOCASCADE:
-            self.hard_delete_cascade_policy_action(**kwargs)
+            return self.hard_delete_cascade_policy_action(**kwargs)
         elif current_policy == SOFT_DELETE_CASCADE:
-            self.soft_delete_cascade_policy_action(**kwargs)
+            return self.soft_delete_cascade_policy_action(**kwargs)
 
     def soft_delete_policy_action(self, **kwargs):
         # Only soft-delete the object, marking it as deleted.
         setattr(self, FIELD_NAME, timezone.now())
+
+        # is_cascade shouldn't be in kwargs when calling save method.
+        if kwargs.pop('is_cascade', False):
+            setattr(self, DELETED_BY_CASCADE_FIELD_NAME, True)
+
         using = kwargs.get('using') or router.db_for_write(self.__class__, instance=self)
         # send pre_softdelete signal
         pre_softdelete.send(sender=self.__class__, instance=self, using=using)
@@ -165,25 +192,30 @@ class SafeDeleteModel(models.Model):
         # send softdelete signal
         post_softdelete.send(sender=self.__class__, instance=self, using=using)
 
+        return (1, {self._meta.label: 1})
+
     def hard_delete_policy_action(self, **kwargs):
         # Normally hard-delete the object.
-        super(SafeDeleteModel, self).delete()
+        return super(SafeDeleteModel, self).delete()
 
     def hard_delete_cascade_policy_action(self, **kwargs):
         # Hard-delete the object only if nothing would be deleted with it
         if not can_hard_delete(self):
-            self._delete(force_policy=SOFT_DELETE, **kwargs)
+            return self._delete(force_policy=SOFT_DELETE, **kwargs)
         else:
-            self._delete(force_policy=HARD_DELETE, **kwargs)
+            return self._delete(force_policy=HARD_DELETE, **kwargs)
 
     def soft_delete_cascade_policy_action(self, **kwargs):
         # Soft-delete on related objects before
+        deleted_counter = Counter()
         for related in related_objects(self):
             if is_safedelete_cls(related.__class__) and not getattr(related, FIELD_NAME):
-                related.delete(force_policy=SOFT_DELETE, **kwargs)
+                _, delete_response = related.delete(force_policy=SOFT_DELETE, is_cascade=True, **kwargs)
+                deleted_counter.update(delete_response)
 
         # soft-delete the object
-        self._delete(force_policy=SOFT_DELETE, **kwargs)
+        _, delete_response = self._delete(force_policy=SOFT_DELETE, **kwargs)
+        deleted_counter.update(delete_response)
 
         collector = NestedObjects(using=router.db_for_write(self))
         collector.collect([self])
@@ -197,6 +229,8 @@ class SafeDeleteModel(models.Model):
                     collector.using,
                 )
 
+        return sum(deleted_counter.values()), dict(deleted_counter)
+
     @classmethod
     def has_unique_fields(cls):
         """Checks if one of the fields of this model has a unique constraint set (unique=True).
@@ -262,6 +296,7 @@ class SafeDeleteModel(models.Model):
 
 
 SafeDeleteModel.add_to_class(FIELD_NAME, models.DateTimeField(editable=False, null=True))
+SafeDeleteModel.add_to_class(DELETED_BY_CASCADE_FIELD_NAME, models.BooleanField(editable=False, default=False))
 
 
 class SafeDeleteMixin(SafeDeleteModel):
